<?php

/*
 * This file is part of the Qsnh/meedu.
 *
 * (c) XiaoTeng <616896861@qq.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace Addons\No9Theme\Http\Controllers\Api;

use Addons\No9Theme\Constant\Constant;
use App\Http\Controllers\Api\V2\BaseController;
use App\Services\Course\Interfaces\CourseServiceInterface;
use App\Services\Course\Services\CourseService;
use Illuminate\Http\Request;
use App\Businesses\BusinessState;
use App\Constant\FrontendConstant;
use App\Exceptions\SystemException;
use App\Exceptions\ServiceException;
use App\Services\Base\Services\CacheService;
use App\Services\Base\Services\ConfigService;
use App\Services\Order\Services\OrderService;
use App\Services\Base\Interfaces\CacheServiceInterface;
use App\Services\Base\Interfaces\ConfigServiceInterface;
use App\Services\Order\Interfaces\OrderServiceInterface;
use Illuminate\Support\Facades\Auth;

class OrderController extends BaseController
{
    /**
     * @var CourseService
     */
    protected $courseService;

    /**
     * @var OrderService
     */
    protected $orderService;
    /**
     * @var ConfigService
     */
    protected $configService;
    protected $businessState;
    /**
     * @var CacheService
     */
    protected $cacheService;

    public function __construct(
        CourseServiceInterface $courseService,
        OrderServiceInterface $orderService,
        ConfigServiceInterface $configService,
        BusinessState $businessState,
        CacheServiceInterface $cacheService
    )
    {
        $this->orderService = $orderService;
        $this->configService = $configService;
        $this->businessState = $businessState;
        $this->cacheService = $cacheService;
        $this->courseService = $courseService;
    }

    public function courseBuyHandler(Request $request)
    {
        $id = $request->input('course_id');
        $promoCodeId = abs((int)$request->input('promo_code_id'));
        $course = $this->courseService->find($id);
        if ($course['charge'] <= 0) {
            return $this->error(__('course cant buy'));
        }
        $order = $this->orderService->createCourseOrder(Auth::id(), $course, $promoCodeId);

        if ($order['status'] === FrontendConstant::ORDER_PAID) {
            return $this->error([$course['id'], $course['slug']]);
        }

        $paymentScene = $request->input('payment_scene');
        $payment = $request->input('payment_sign');
        return $this->data(['scene' => $paymentScene, 'payment' => $payment, 'order_id' => $order['order_id']]);
    }

    /**
     * @param Request $request
     * @return mixed
     * @throws ServiceException
     * @throws SystemException
     * @throws \Illuminate\Contracts\Container\BindingResolutionException
     */
    public function pay(Request $request)
    {
        $orderId = $request->input('order_id');
        $order = $this->orderService->findUserNoPaid($orderId);

        $scene = $request->input('scene');
        $payment = $request->input('payment');
        if (!$payment) {
            throw new ServiceException(__('payment not exists'));
        }
        $payments = Constant::No9_PAYMENT;
        $paymentMethod = $payments[$payment][$scene] ?? '';
        if (!$paymentMethod) {
            throw new SystemException(__('payment method not exists'));
        }

        // 更新订单的支付方式
        $updateData = [
            'payment' => $payment,
            'payment_method' => $paymentMethod,
        ];
        $this->orderService->change2Paying($order['id'], $updateData);
        $order = array_merge($order, $updateData);

        // 创建远程订单
        $paymentHandler = app()->make($payments[$payment]['handler']);
        $createResult = $paymentHandler->create($order);
        if ($createResult->status === false) {
            throw new SystemException(__('remote order create failed'));
        }

        return $this->data($createResult->data);
    }

    /**
     * @param Request $request
     *
     */
    public function paySuccess(Request $request)
    {
        $orderId = $request->input('out_trade_no', '');
        $order = $this->orderService->findUser($orderId);

        return $this->data($order);
    }

    /**
     * @param $orderId
     * @throws \App\Exceptions\ServiceException
     */
    public function wechat($orderId)
    {
        try {
            $order = $this->orderService->findUser($orderId);
            $needPaidTotal = $this->businessState->calculateOrderNeedPaidSum($order);

            $wechatData = $this->cacheService->pull(sprintf(FrontendConstant::PAYMENT_WECHAT_PAY_CACHE_KEY, $order['order_id']));
            if (!$wechatData) {
                $this->orderService->cancel($order['id']);

                return $this->error(__('error'));
            }

            $qrcodeUrl = $wechatData['code_url'];
        } catch (\Exception $e) {
            return $this->error($e->getMessage());
        }

        return $this->data(['qrcodeUrl' => $qrcodeUrl, 'order' => $order, 'needPaidTotal' => $needPaidTotal]);
    }

    /**
     * @param $orderId
     */
    public function handPay($orderId)
    {
        $order = $this->orderService->findUser($orderId);
        $needPaidTotal = $this->businessState->calculateOrderNeedPaidSum($order);
        $intro = $this->configService->getHandPayIntroducation();
        return $this->data(compact('order', 'intro', 'needPaidTotal'));
    }
}
