<?php

/*
 * This file is part of the Qsnh/meedu.
 *
 * (c) XiaoTeng <616896861@qq.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace Addons\No9Theme;

use Addons\No9Theme\Constant\Constant;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\ServiceProvider;
use Addons\No9Theme\Commands\AppCommand;

class MainServiceProvider extends ServiceProvider
{
    protected $listen = [

    ];

    public function boot()
    {
        // 注册视图命名空间
        $this->loadViewsFrom(__DIR__ . '/resources/views', Constant::APP_SIGN);
        // 注册路由
        $this->mapApiRoutes();
        $this->mapWebRoutes();
        $this->mapBackendApiRoutes();
        // 注册命令
        $this->commands([
            AppCommand::class,
        ]);

        // 事件注册
        foreach ($this->listen as $event => $listeners) {
            foreach ($listeners as $listener) {
                Event::listen($event, $listener);
            }
        }
    }

    public function register()
    {
    }

    protected function mapWebRoutes()
    {
        Route::middleware(['web', 'global.share'])
            ->group(__DIR__ . '/routes/web.php');
    }

    protected function mapApiRoutes()
    {
        Route::prefix('/api')
            ->middleware('api')
            ->group(__DIR__ . '/routes/api.php');
    }

    protected function mapBackendApiRoutes()
    {
        Route::prefix('/backend/api')
            ->middleware(['api'])
            ->group(__DIR__ . '/routes/backend.php');
    }
}
